*Title: 'Select up to 3 solvents to maximise the solubility of Ibuprofen
*Nmax=3
*Miscibility constraints are included
*==============================================================================
SETS

s 'solvents'
/methanol, ethanol, 2-propanol, acetone, mibk, ethylacetate,
chloroform, toluene, water/

i 'components in mixture'
/ibuprofen, s1, s2, s3/

ii(i) 'selected solvent'
/s1, s2, s3/


k 'groups'
/ch3, ch2, ch, ach, acch3, acch2, acch, oh, ch3oh, h2o, ch3co, ch3coo, cooh, chcl3/

alias(k,m);
alias(k,p);
alias(s,ss);


PARAMETERS
Hm 'enthalpy of fusion of ibuprofen' /25500/
Tm 'melting point of ibuprofen' /347.15/
T 'system temperature' /300/
Rg 'gas constant' /8.3144/


R(k) 'Van der Waals volume of group k'
/ch3             0.9011
ch2              0.6744
ch               0.4469
ach              0.5313
acch3            1.2663
acch2            1.0396
acch             0.8121
oh               1.0000
ch3oh            1.4311
h2o              0.9200
ch3co            1.6724
ch3coo           1.9031
cooh             1.3013
chcl3            2.8700 /

Q(k) 'Van der Waals surface area of group k'
/ch3             0.848
ch2              0.540
ch               0.228
ach              0.400
acch3            0.968
acch2            0.660
acch             0.348
oh               1.200
ch3oh            1.432
h2o              1.400
ch3co            1.488
ch3coo           1.728
cooh             1.224
chcl3            2.410 /


table v(s,k) 'number of groups in molecule i'
                 ch3     ch2     ch      ach     acch3   acch2   acch    oh      ch3oh   h2o     ch3co   ch3coo  cooh    chcl3

methanol                                                                         1
ethanol          1       1                                               1
2-propanol       2       1                                               1
acetone          1                                                                               1
mibk             2       1       1                                                               1
ethylacetate     1       1                                                                               1
chloroform                                                                                                               1
toluene                                  5       1
water                                                                                    1


table vib(i,k)'identity of ibuprofen'
             ch3     ch ach      acch2 acch   cooh
ibuprofen     3       1  4         1    1       1      ;


                                                                                                                          ;
*===============================================================================
*group interaction parameters
*===============================================================================
table a(k,m) 'group interaction parameters'
         ch3     ch2     ch      ach     acch3   acch2   acch    oh      ch3oh   h2o     ch3co   ch3coo  cooh    chcl3
ch3      0       0       0       61.13   76.5    76.5    76.5    986.5   697.2   1318    476.4   232.1   663.5   24.9
ch2      0       0       0       61.13   76.5    76.5    76.5    986.5   697.2   1318    476.4   232.1   663.5   24.9
ch       0       0       0       61.13   76.5    76.5    76.5    986.5   697.2   1318    476.4   232.1   663.5   24.9
ach      -11.12  -11.12  -11.12  0       167     167     167     636.1   637.4   903.8   25.77   5.994   537.4   -231.9
acch3    -69.7   -69.7   -69.7   -146.8  0       0       0       803.2   603.3   5695    -52.1   5688    872.3   -80.25
acch2    -69.7   -69.7   -69.7   -146.8  0       0       0       803.2   603.3   5695    -52.1   5688    872.3   -80.25
acch     -69.7   -69.7   -69.7   -146.8  0       0       0       803.2   603.3   5695    -52.1   5688    872.3   -80.25
oh       156.4   156.4   156.4   89.6    25.82   25.82   25.82   0       -137.1  353.5   84      101.1   199     -98.12
ch3oh    16.51   16.51   16.51   -50     -44.5   -44.5   -44.5   249.1   0       -181    23.39   -10.72  -202    -139.4
h2o      300     300     300     362.3   377.6   377.6   377.6   -229.1  289.6   0       -195.4  72.87   -14.09  353.7
ch3co    26.76   26.76   26.76   140.1   365.8   365.8   365.8   164.5   108.7   472.5   0       -213.7  669.4   -354.6
ch3coo   114.8   114.8   114.8   85.84   -170    -170    -170    245.4   249.6   200.8   372.2   0       660.2   -209.7
cooh     315.3   315.3   315.3   62.32   89.86   89.86   89.86   -151    339.8   -66.17  -297.8  -256.3  0       39.63
chcl3    36.7    36.7    36.7    288.5   69.9    69.9    69.9    742.1   649.1   826.8   552.1   176.5   504.2   0
                                                                                                                    ;



parameter ps(k,m);
ps(k,m)= exp(-a(k,m)/T);
parameter nib(i,k);
nib('ibuprofen',k)= vib('ibuprofen',k);
parameter qib;
qib = sum(k,nib('ibuprofen',k)*Q(k));
parameter rib;
rib = sum(k,nib('ibuprofen',k)*R(k));
parameter qs(s);
qs(s) = sum(k,v(s,k)*Q(k));
parameter rs(s);
rs(s) = sum(k,v(s,k)*R(k));
parameter eib(i,k);
eib('ibuprofen',k)=vib('ibuprofen',k)*Q(k)/qib;
parameter bib(i,k);
bib('ibuprofen',k)=sum(m,eib('ibuprofen',m)*ps(m,k));


POSITIVE VARIABLES

x(i)             'liquid phase mole fraction of component i'
rc(i)            'van der waals volume of component i'
qc(i)            'van der waals area of component i'
J(i),L(i)

th(k)
w(k)

x12(ii)
b12(ii,k)
th12(k)
w12(k)
sumr12
sumq12

x13(ii)
th13(k)
w13(k)
sumr13
sumq13

x23(ii)
th23(k)
w23(k)
sumr23
sumq23


FREE VARIABLES
z            'objective function'

lng(i)       'natural log of activity coefficient of ibuprofen'
lngc(i)      'natural log of combinatorial activity coefficient of ibuprofen'
lngr(i)      'natural log of residual activity coefficient of ibuprofen'

d_lngc12(ii)  'differention of combinatorial act_coeff'
d_lngr12(ii)  'differention of residual act_coeff'
d_lng12(ii)   'differentiation of act_coef wrt to x(ii)'
d_th12(k)     'diff of intermediate'
d_w12(k)      'diff of intermediate'
a12(k)     'intermediate'
c12(k)     'intermediate'

d_lngc13(ii)  'differention of combinatorial act_coeff'
d_lngr13(ii)  'differention of residual act_coeff'
d_lng13(ii)   'differentiation of act_coef wrt to x(ii)'
d_th13(k)     'diff of intermediate'
d_w13(k)      'diff of intermediate'
a13(k)     'intermediate'
c13(k)     'intermediate'

d_lngc23(ii)  'differention of combinatorial act_coeff'
d_lngr23(ii)  'differention of residual act_coeff'
d_lng23(ii)   'differentiation of act_coef wrt to x(ii)'
d_th23(k)     'diff of intermediate'
d_w23(k)      'diff of intermediate'
a23(k)     'intermediate'
c23(k)     'intermediate'



BINARY VARIABLES
y(ii,s)     'selected solvents'
Y1          '1 solvent  choosen'
Y2          '2 solvents choosen'
Y3          '3 solvents choosen'

INTEGER VARIABLE
n(i,k)    'number of groups k in component i' ;


EQUATIONS
eq_z                   'objective function maximise solubility of ibuprofen'
eq_x                   'mole fraction constrain'
eq_n(ii,k)
eq_qc(ii)
eq_rc(ii)
eq_J,eq_L
eq_lngc
eq_th(k)
eq_w(k)
eq_lngr
eq_lng
eq_solub
*equations for logical conditions
logic1,logic2a,logic2b(ii),logic3(s),logic4(s),logic5(s)
logic6,logic7,logic8,logic9(s)
logic10(s,ss),logic11(s,ss),logic12(s,ss)
logic13,logic14,logic15a,logic15b,logic16a,logic16b
*equations for miscibility constraints
eq_x12a,eq_x12b,eq_x12c,eq_x12d
eq_sumr12a,eq_sumr12b,eq_sumq12a,eq_sumq12b,eq_dlngc12a,eq_dlngc12b,eq_th12(k),eq_dth12(k),eq_w12(k),eq_dw12(k),eq_a12(k),eq_b12a(k),eq_b12b(k),eq_b12c(k),eq_b12d(k),eq_c12(k),eq_dlngr12a,eq_dlngr12b,eq_dlng12a,eq_dlng12b,eq_misc12
eq_x13a,eq_x13b,eq_x13c,eq_x13d
eq_sumr13a,eq_sumr13b,eq_sumq13a,eq_sumq13b,eq_dlngc13a,eq_dlngc13b,eq_th13(k),eq_dth13(k),eq_w13(k),eq_dw13(k),eq_a13(k),eq_c13(k),eq_dlngr13a,eq_dlngr13b,eq_dlng13a,eq_dlng13b,eq_misc13
eq_x23a,eq_x23b,eq_x23c,eq_x23d    
eq_sumr23a,eq_sumr23b,eq_sumq23a,eq_sumq23b,eq_dlngc23a,eq_dlngc23b,eq_th23(k),eq_dth23(k),eq_w23(k),eq_dw23(k),eq_a23(k),eq_c23(k),eq_dlngr23a,eq_dlngr23b,eq_dlng23a,eq_dlng23b,eq_misc23
;



********************Only one disjunction chosen*********************************
logic1..Y1+Y2+Y3=e=1;

***********At most Nmax solvents...(i.e. at most 3 solvents)********************
*at least one solvent must be selected
logic2a..
sum(s,y('s1',s))=e=1;
*select at most 2 more solvents
logic2b(ii)$(not sameas(ii,'s1'))..
sum(s,y(ii,s))=l=1;

*******logical conditions between # of solvents & choise of solvents************
logic3(s)..Y1+y('s2',s)=l=1;

logic4(s)..Y1+y('s3',s)=l=1;

logic5(s)..Y2+y('s3',s)=l=1;

logic6..Y2=l=sum(s,y('s2',s));

logic7..Y3=l=sum(s,y('s2',s));

logic8..Y3=l=sum(s,y('s3',s));

*****************************solvent ordering***********************************
logic9(s)..y('s1',s)+y('s2',s)+y('s3',s)=l=1;

logic10(s,ss)$(ord(ss) <ord(s))..y('s1',s)+y('s2',s-ord(ss))=l=1;

logic11(s,ss)$(ord(ss) <ord(s))..y('s1',s)+y('s3',s-ord(ss))=l=1;

logic12(s,ss)$(ord(ss) <ord(s))..y('s2',s)+y('s3',s-ord(ss))=l=1;

*****************reformulation of composition***********************************
logic13..x('ibuprofen')=g=0.001;

logic14..x('s1')=g=0.001;

logic15a..x('s2')=g=0.001*(1-Y1);

logic15b..x('s2')=l=0.999*(1-Y1);

logic16a..x('s3')=g=0.001*Y3;

logic16b..x('s3')=l=0.999*Y3 ;

*******************************UNIFAC*******************************************
eq_z..
z=e=x('ibuprofen');

eq_x..
sum(i,x(i))=e=1;

eq_n(ii,k)..n(ii,k)=e=sum(s,v(s,k)*y(ii,s));

eq_qc(ii)..qc(ii)=e= sum(s,qs(s)*y(ii,s));

eq_rc(ii)..rc(ii)=e= sum(s,rs(s)*y(ii,s));

*combinatorial part of activity coefficient
eq_J..J('ibuprofen')*sum(i,x(i)*rc(i))=e=rib;
eq_L..L('ibuprofen')*sum(i,x(i)*qc(i))=e=qib;

eq_lngc..
lngc('ibuprofen')=e=1-J('ibuprofen')+log(J('ibuprofen'))-5*qib*(1-J('ibuprofen')/L('ibuprofen')+log(J('ibuprofen'))-log(L('ibuprofen')));

*residual part of activity coefficient
eq_th(k)..th(k)*sum(i,x(i)*qc(i))=e=sum(i,x(i)*Q(k)*n(i,k));
eq_w(k)..w(k)=e=sum(m,th(m)*ps(m,k));

eq_lngr..
lngr('ibuprofen')=e=qib*(1-sum(k,th(k)*bib('ibuprofen',k)/w(k)-eib('ibuprofen',k)*(log(bib('ibuprofen',k))-log(w(k)))));

*activity coefficient
eq_lng..
lng('ibuprofen') =e=  lngc('ibuprofen') + lngr('ibuprofen');

*ibuprofen solubility constraint
eq_solub..
log(x('ibuprofen'))+ lng('ibuprofen') =e= Hm/Rg *(1/Tm-1/T);

*****************Miscibility constraints between s1 & s2************************
eq_x12a..x12('s1')*(x('s1')+x('s2'))-x('s1')=l=10*Y1;
eq_x12b..x12('s1')*(x('s1')+x('s2'))-x('s1')=g=-10*Y1;
eq_x12c..x12('s1')+x12('s2')-1=l=10*Y1;
eq_x12d..x12('s1')+x12('s2')-1=g=-10*Y1;

eq_sumr12a..-sumr12+x12('s1')*rc('s1')+(1-x12('s1'))*rc('s2')=l=10*Y1;
eq_sumr12b..-sumr12+x12('s1')*rc('s1')+(1-x12('s1'))*rc('s2')=g=-10*Y1;

eq_sumq12a..-sumq12+x12('s1')*qc('s1')+(1-x12('s1'))*qc('s2')=l=10*Y1;
eq_sumq12b..-sumq12+x12('s1')*qc('s1')+(1-x12('s1'))*qc('s2')=g=-10*Y1;

*differ of combinatorial activity coefficient, i.e. dlngc/dx(s1)
eq_dlngc12a..-d_lngc12('s1')*power(sumr12,2)*sumq12+5*qc('s1')*(qc('s2')-qc('s1'))*power(sumr12,2)
                                        +(rc('s1')-rc('s2'))*(5*qc('s1')-1)*sumr12*sumq12
                                        +5*rc('s1')*(qc('s1')-qc('s2'))*sumr12*sumq12
                                        +((rc('s1')-rc('s2'))*(rc('s1')-5*rc('s1')*(sumq12)))*sumq12=l=100*Y1;
eq_dlngc12b..-d_lngc12('s1')*power(sumr12,2)*sumq12+5*qc('s1')*(qc('s2')-qc('s1'))*power(sumr12,2)
                                        +(rc('s1')-rc('s2'))*(5*qc('s1')-1)*sumr12*sumq12
                                        +5*rc('s1')*(qc('s1')-qc('s2'))*sumr12*sumq12
                                        +((rc('s1')-rc('s2'))*(rc('s1')-5*rc('s1')*(sumq12)))*sumq12=g=-100*Y1;

eq_th12(k)..
th12(k)*sumq12=e=x12('s1')*Q(k)*n('s1',k)+(1-x12('s1'))*Q(k)*n('s2',k);

eq_w12(k)..w12(k)=e=sum(m,th12(m)*ps(m,k));

eq_dth12(k)..
d_th12(k)*power(sumq12,2)=e=((Q(k)*n('s1',k)- Q(k)*n('s2',k))*sumq12)-(x12('s1')*Q(k)*n('s1',k)+ x12('s2')*Q(k)*n('s2',k))*(qc('s1')-qc('s2'));

eq_dw12(k)..d_w12(k)=e=sum(m,ps(m,k)*d_th12(m));

eq_a12(k)..a12(k)*power(w12(k),2)=e=d_th12(k)*w12(k)-th12(k)*d_w12(k);

eq_b12a(k)..b12('s1',k)-sum(m,Q(m)*n('s1',m)*ps(m,k))=l=100*Y1;
eq_b12b(k)..b12('s1',k)-sum(m,Q(m)*n('s1',m)*ps(m,k))=g=-100*Y1;
eq_b12c(k)..b12('s2',k)-sum(m,Q(m)*n('s2',m)*ps(m,k))=l=100*(1-Y3);
eq_b12d(k)..b12('s2',k)-sum(m,Q(m)*n('s2',m)*ps(m,k))=g=-100*(1-Y3);

eq_c12(k)..c12(k)*w12(k)=e=d_w12(k);

*differ of residual activity coefficient, i.e. dlngr/dx(s1)
*simplier form, not the general form of unifac
eq_dlngr12a..d_lngr12('s1')+sum(k,a12(k)*b12('s1',k)+Q(k)*n('s1',k)*c12(k))=l=100*Y1;
eq_dlngr12b..d_lngr12('s1')+sum(k,a12(k)*b12('s1',k)+Q(k)*n('s1',k)*c12(k))=g=-100*Y1;

*differ of activity coefficient,i.e. dlng/dx(s1)
eq_dlng12a..d_lng12('s1')-d_lngc12('s1')-d_lngr12('s1')=l=100*Y1;
eq_dlng12b..d_lng12('s1')-d_lngc12('s1')-d_lngr12('s1')=g=-100*Y1;

eq_misc12..d_lng12('s1')*x12('s1')+1=g=-100*Y1;

*****************Miscibility constraints between s1 & s3************************
eq_x13a..x13('s1')*(x('s1')+x('s3'))-x('s1')=l=10*(1-Y3);
eq_x13b..x13('s1')*(x('s1')+x('s3'))-x('s1')=g=-10*(1-Y3);
eq_x13c..x13('s1')+x13('s3')-1=l=10*(1-Y3);
eq_x13d..x13('s1')+x13('s3')-1=g=-10*(1-Y3);

eq_sumr13a..-sumr13+x13('s1')*rc('s1')+(1-x13('s1'))*rc('s3')=l=10*(1-Y3);
eq_sumr13b..-sumr13+x13('s1')*rc('s1')+(1-x13('s1'))*rc('s3')=g=-10*(1-Y3);

eq_sumq13a..-sumq13+x13('s1')*qc('s1')+(1-x13('s1'))*qc('s3')=l=100*(1-Y3);
eq_sumq13b..-sumq13+x13('s1')*qc('s1')+(1-x13('s1'))*qc('s3')=g=-100*(1-Y3);

*differ of combinatorial activity coefficient, i.e. dlngc/dx(s1)
eq_dlngc13a..-d_lngc13('s1')*power(sumr13,2)*sumq13+5*qc('s1')*(qc('s3')-qc('s1'))*power(sumr13,2)
                                        +(rc('s1')-rc('s3'))*(5*qc('s1')-1)*sumr13*sumq13
                                        +5*rc('s1')*(qc('s1')-qc('s3'))*sumr13*sumq13
                                        +((rc('s1')-rc('s3'))*(rc('s1')-5*rc('s1')*(sumq13)))*sumq13=l=100*(1-Y3);
eq_dlngc13b..-d_lngc13('s1')*power(sumr13,2)*sumq13+5*qc('s1')*(qc('s3')-qc('s1'))*power(sumr13,2)
                                        +(rc('s1')-rc('s3'))*(5*qc('s1')-1)*sumr13*sumq13
                                        +5*rc('s1')*(qc('s1')-qc('s3'))*sumr13*sumq13
                                        +((rc('s1')-rc('s3'))*(rc('s1')-5*rc('s1')*(sumq13)))*sumq13=g=-100*(1-Y3);

eq_th13(k)..
th13(k)*sumq13=e=x13('s1')*Q(k)*n('s1',k)+(1-x13('s1'))*Q(k)*n('s3',k);

eq_w13(k)..w13(k)=e=sum(m,th13(m)*ps(m,k));

eq_dth13(k)..
d_th13(k)*power(sumq13,2)=e=((Q(k)*n('s1',k)- Q(k)*n('s3',k))*sumq13)-(x13('s1')*Q(k)*n('s1',k)+ x13('s3')*Q(k)*n('s3',k))*(qc('s1')-qc('s3'));

eq_dw13(k)..d_w13(k)=e=sum(m,ps(m,k)*d_th13(m));

eq_a13(k)..a13(k)*power(w13(k),2)=e=d_th13(k)*w13(k)-th13(k)*d_w13(k);
eq_c13(k)..c13(k)*w13(k)=e=d_w13(k);

*differ of residual activity coefficient, i.e. dlngr/dx(s1)
*simplier form, not the general form of unifac
eq_dlngr13a..d_lngr13('s1')+sum(k,a13(k)*b12('s1',k)+Q(k)*n('s1',k)*c13(k))=l=100*(1-Y3);
eq_dlngr13b..d_lngr13('s1')+sum(k,a13(k)*b12('s1',k)+Q(k)*n('s1',k)*c13(k))=g=-100*(1-Y3);

*differ of activity coefficient,i.e. dlng/dx(s1)
eq_dlng13a..d_lng13('s1')-d_lngc13('s1')-d_lngr13('s1')=l=100*(1-Y3);
eq_dlng13b..d_lng13('s1')-d_lngc13('s1')-d_lngr13('s1')=g=-100*(1-Y3);

eq_misc13..d_lng13('s1')*x13('s1')+1=g=-100*(1-Y3);

*****************Miscibility constraints between s2 & s3************************
eq_x23a..x23('s2')*(x('s2')+x('s3'))-x('s2')=l=10*(1-Y3);
eq_x23b..x23('s2')*(x('s2')+x('s3'))-x('s2')=g=-10*(1-Y3);
eq_x23c..x23('s2')+x23('s3')-1=l=10*(1-Y3);
eq_x23d..x23('s2')+x23('s3')-1=g=-10*(1-Y3);

eq_sumr23a..-sumr23+x23('s2')*rc('s2')+(1-x23('s2'))*rc('s3')=l=10*(1-Y3);
eq_sumr23b..-sumr23+x23('s2')*rc('s2')+(1-x23('s2'))*rc('s3')=g=-10*(1-Y3);

eq_sumq23a..-sumq23+x23('s2')*qc('s2')+(1-x23('s2'))*qc('s3')=l=100*(1-Y3);
eq_sumq23b..-sumq23+x23('s2')*qc('s2')+(1-x23('s2'))*qc('s3')=g=-100*(1-Y3);

*differ of combinatorial activity coefficient, i.e. dlngc/dx(s1)
eq_dlngc23a..-d_lngc23('s2')*power(sumr23,2)*sumq23+5*qc('s2')*(qc('s3')-qc('s2'))*power(sumr23,2)
                                        +(rc('s2')-rc('s3'))*(5*qc('s2')-1)*sumr23*sumq23
                                        +5*rc('s2')*(qc('s2')-qc('s3'))*sumr23*sumq23
                                        +((rc('s2')-rc('s3'))*(rc('s2')-5*rc('s2')*(sumq23)))*sumq23=l=100*(1-Y3);
eq_dlngc23b..-d_lngc23('s2')*power(sumr23,2)*sumq23+5*qc('s2')*(qc('s3')-qc('s2'))*power(sumr23,2)
                                        +(rc('s2')-rc('s3'))*(5*qc('s2')-1)*sumr23*sumq23
                                        +5*rc('s2')*(qc('s2')-qc('s3'))*sumr23*sumq23
                                        +((rc('s2')-rc('s3'))*(rc('s2')-5*rc('s2')*(sumq23)))*sumq23=g=-100*(1-Y3);

eq_th23(k)..
th23(k)*sumq23=e=x23('s2')*Q(k)*n('s2',k)+(1-x23('s2'))*Q(k)*n('s3',k);

eq_w23(k)..w23(k)=e=sum(m,th23(m)*ps(m,k));

eq_dth23(k)..
d_th23(k)*power(sumq23,2)=e=((Q(k)*n('s2',k)- Q(k)*n('s3',k))*sumq23)-(x23('s2')*Q(k)*n('s2',k)+ x23('s3')*Q(k)*n('s3',k))*(qc('s2')-qc('s3'));

eq_dw23(k)..d_w23(k)=e=sum(m,ps(m,k)*d_th23(m));

eq_a23(k)..a23(k)*power(w23(k),2)=e=d_th23(k)*w23(k)-th23(k)*d_w23(k);
eq_c23(k)..c23(k)*w23(k)=e=d_w23(k);

*differ of residual activity coefficient, i.e. dlngr/dx(s2)
*simplier form, not the general form of unifac
eq_dlngr23a..d_lngr23('s2')+sum(k,a23(k)*b12('s2',k)+Q(k)*n('s2',k)*c23(k))=l=100*(1-Y3);
eq_dlngr23b..d_lngr23('s2')+sum(k,a23(k)*b12('s2',k)+Q(k)*n('s2',k)*c23(k))=g=-100*(1-Y3);

*differ of activity coefficient,i.e. dlng/dx(s1)
eq_dlng23a..d_lng23('s2')-d_lngc23('s2')-d_lngr23('s2')=l=100*(1-Y3);
eq_dlng23b..d_lng23('s2')-d_lngc23('s2')-d_lngr23('s2')=g=-100*(1-Y3);

eq_misc23..d_lng23('s2')*x23('s2')+1=g=-100*(1-Y3);

*******************************BOUNDS*******************************************
*bounds on optmization variables
x.up(i)=1;
x.lo('ibuprofen')=0.001;
x.lo('s1')=0.001;
x.lo('s2')=0;
x.lo('s3')=0;

x12.up(ii)=1;
x12.lo('s1')=0.001;
x12.lo('s2')=0;
x13.up(ii)=1;
x13.lo('s1')=0.001;
x13.lo('s3')=0;
x23.up(ii)=1;
x23.lo(ii)=0;

n.up(ii,k)=6;

*variable bounds
rc.lo('s1')= 0.1;
rc.lo(ii)$(not sameas(ii,'s1'))= 0;
rc.up(i)= 10;
qc.lo('s1')= 0.1;
qc.lo(ii)$(not sameas(ii,'s1'))= 0;
qc.up(i)= 10;

J.lo('ibuprofen')=0.1;
J.up('ibuprofen')=10;
L.lo('ibuprofen')=0.1;
L.up('ibuprofen')=10;

th.up(k)=1;
th.lo(k)=0;
w.up(k)=10;
w.lo(k)=0;

lngc.lo('ibuprofen')=-100;
lngc.up('ibuprofen')=100;
lngr.lo('ibuprofen')=-100;
lngr.up('ibuprofen')=100;
lng.lo('ibuprofen') = -100;
lng.up('ibuprofen') = 100;

*bounds for imiscibility calculations
*s1 & s2 bounds
*================================================================================
th12.up(k)=1;
th12.lo(k)=0;
w12.up(k)=10;
w12.lo(k)=0.0001;

sumr12.lo=0.1;
sumr12.up=10;
sumq12.lo=0.1;
sumq12.up=10;

d_lngc12.lo('s1')=-100;
d_lngc12.up('s1')=100;

d_th12.lo(k)=-100;
d_th12.up(k)=100;

d_w12.lo(k)=-100;
d_w12.up(k)= 100;

a12.lo(k)=-100;
a12.up(k)= 100;

b12.up(ii,k)=100 ;

c12.lo(k)=-100;
c12.up(k)= 100;

d_lngr12.lo('s1')=-100;
d_lngr12.up('s1')=100;

d_lng12.lo('s1')=-100;
d_lng12.up('s1')=100;

*s1 & s3 bounds
*================================================================================
th13.up(k)=1;
th13.lo(k)=0;
w13.up(k)=10;
w13.lo(k)=0.0001;

sumr13.lo=0.1;
sumr13.up=10;
sumq13.lo=0.1;
sumq13.up=10;

d_lngc13.lo('s1')=-100;
d_lngc13.up('s1')=100;

d_th13.lo(k)=-100;
d_th13.up(k)=100;

d_w13.lo(k)=-100;
d_w13.up(k)= 100;

a13.lo(k)=-100;
a13.up(k)= 100;

c13.lo(k)=-100;
c13.up(k)= 100;

d_lngr13.lo('s1')=-100;
d_lngr13.up('s1')=100;

d_lng13.lo('s1')=-100;
d_lng13.up('s1')=100;

*s2 & s3 bounds
*================================================================================
th23.up(k)=1;
th23.lo(k)=0;
w23.up(k)=10;
w23.lo(k)=0;

sumr23.lo=0;
sumr23.up=10;
sumq23.lo=0;
sumq23.up=10;

d_lngc23.lo('s2')=-100;
d_lngc23.up('s2')=100;

d_th23.lo(k)=-100;
d_th23.up(k)=100;

d_w23.lo(k)=-100;
d_w23.up(k)= 100;

a23.lo(k)=-100;
a23.up(k)= 100;

c23.lo(k)=-100;
c23.up(k)= 100;

d_lngr23.lo('s2')=-100;
d_lngr23.up('s2')=100;

d_lng23.lo('s2')=-100;
d_lng23.up('s2')=100;


*********************INITIAL POINTS*********************************************

*fixing of ibuprofen identity
qc.fx('ibuprofen')=qib;
rc.fx('ibuprofen')=rib;
n.fx('ibuprofen',k)=vib('ibuprofen',k);

*initilaization  of optimal solvent
Y1.l=0;
Y2.l=1;
Y3.l=1-Y1.l-Y2.l;

x.l('ibuprofen')=0.3;
x.l('s2')=0.2;
x.l('s3')=0.2;
x.l('s1')=1-x.l('ibuprofen')-x.l('s2')-x.l('s3');

y.l('s1','methanol')=1;
y.l('s2','chloroform')=1;
y.l('s3','toluene')=1;

z.l=x.l('ibuprofen');

n.l(ii,k)=sum(s,v(s,k)*y.l(ii,s));

rc.l(ii)= sum(s,rs(s)*y.l(ii,s));
qc.l(ii)= sum(s,qs(s)*y.l(ii,s));

J.l('ibuprofen')=rib/sum(i,x.l(i)*rc.l(i));
L.l('ibuprofen')=qib/sum(i,x.l(i)*rc.l(i));

lngc.l('ibuprofen')=1-J.l('ibuprofen')+log(J.l('ibuprofen'))-5*qib*(1-J.l('ibuprofen')/L.l('ibuprofen')+log(J.l('ibuprofen'))-log(L.l('ibuprofen')));

th.l(k)=sum(i,x.l(i)*Q(k)*n.l(i,k))/sum(i,x.l(i)*qc.l(i));
w.l(k)=sum(m,th.l(m)*ps(m,k));

lngr.l('ibuprofen')=qib*(1-sum(k,th.l(k)*bib('ibuprofen',k)/w.l(k)-eib('ibuprofen',k)*(log(bib('ibuprofen',k))-log(w.l(k)))));

lng.l('ibuprofen') = lngc.l('ibuprofen') + lngr.l('ibuprofen');

*initialisation of imiscibility calculations(s1,s2)
x12.l('s1')=x.l('s1')/(x.l('s1')+x.l('s2'));
x12.l('s2')=1-x12.l('s1');

sumr12.l=x12.l('s1')*rc.l('s1')+(1-x12.l('s1'))*rc.l('s2');
sumq12.l=x12.l('s1')*qc.l('s1')+(1-x12.l('s1'))*qc.l('s2');

d_lngc12.l('s1')=5*qc.l('s1')*(qc.l('s2')-qc.l('s1'))/sumq12.l
                  +(rc.l('s1')-rc.l('s2'))*(5*qc.l('s1')-1)/sumr12.l
                  +5*rc.l('s1')*(qc.l('s1')-qc.l('s2'))/sumr12.l
                  +((rc.l('s1')-rc.l('s2'))*(rc.l('s1')-5*rc.l('s1')*(sumq12.l)))/power(sumr12.l,2);

th12.l(k)=(x12.l('s1')*Q(k)*n.l('s1',k)+(1-x12.l('s1'))*Q(k)*n.l('s2',k))/sumq12.l;

w12.l(k)=sum(m,th12.l(m)*ps(m,k));

d_th12.l(k)=((Q(k)*n.l('s1',k)- Q(k)*n.l('s2',k))*sumq12.l-(x12.l('s1')*Q(k)*n.l('s1',k)+ x12.l('s2')*Q(k)*n.l('s2',k))*(qc.l('s1')-qc.l('s2')))/power(sumq12.l,2);

d_w12.l(k)=sum(m,ps(m,k)*d_th12.l(m));

a12.l(k)=(d_th12.l(k)*w12.l(k)-th12.l(k)*d_w12.l(k))/power(w12.l(k),2);
b12.l(ii,k)=sum(m,Q(m)*n.l(ii,m)*ps(m,k));
c12.l(k)=d_w12.l(k)/w12.l(k);

d_lngr12.l('s1')=-sum(k,a12.l(k)*b12.l('s1',k)+Q(k)*n.l('s1',k)*c12.l(k));

d_lng12.l('s1')=d_lngc12.l('s1')+d_lngr12.l('s1');

*initialisation of imiscibility calculations(s1,s3)
x13.l('s1')=x.l('s1')/(x.l('s1')+x.l('s3'));
x13.l('s3')=1-x13.l('s1');

sumr13.l=x13.l('s1')*rc.l('s1')+(1-x13.l('s1'))*rc.l('s3');
sumq13.l=x13.l('s1')*qc.l('s1')+(1-x13.l('s1'))*qc.l('s3');

d_lngc13.l('s1')=5*qc.l('s1')*(qc.l('s3')-qc.l('s1'))/sumq13.l
                  +(rc.l('s1')-rc.l('s3'))*(5*qc.l('s1')-1)/sumr13.l
                  +5*rc.l('s1')*(qc.l('s1')-qc.l('s3'))/sumr13.l
                  +((rc.l('s1')-rc.l('s3'))*(rc.l('s1')-5*rc.l('s1')*(sumq13.l)))/power(sumr13.l,2);

th13.l(k)=(x13.l('s1')*Q(k)*n.l('s1',k)+(1-x13.l('s1'))*Q(k)*n.l('s3',k))/sumq13.l;

w13.l(k)=sum(m,th13.l(m)*ps(m,k));

d_th13.l(k)=((Q(k)*n.l('s1',k)- Q(k)*n.l('s3',k))*sumq13.l-(x13.l('s1')*Q(k)*n.l('s1',k)+ x13.l('s3')*Q(k)*n.l('s3',k))*(qc.l('s1')-qc.l('s3')))/power(sumq13.l,2);

d_w13.l(k)=sum(m,ps(m,k)*d_th13.l(m));

a13.l(k)=(d_th13.l(k)*w13.l(k)-th13.l(k)*d_w13.l(k))/power(w13.l(k),2);
c13.l(k)=d_w13.l(k)/w13.l(k);

d_lngr13.l('s1')=-sum(k,a13.l(k)*b12.l('s1',k)+Q(k)*n.l('s1',k)*c13.l(k));

d_lng13.l('s1')=d_lngc13.l('s1')+d_lngr13.l('s1');

*initialisation of imiscibility calculations(s2,s3)
x23.l('s2')=x.l('s2')/(x.l('s2')+x.l('s3'));
x23.l('s3')=1-x23.l('s2');

sumr23.l=x23.l('s2')*rc.l('s2')+(1-x23.l('s2'))*rc.l('s3');
sumq23.l=x23.l('s2')*qc.l('s2')+(1-x23.l('s2'))*qc.l('s3');

d_lngc23.l('s2')=5*qc.l('s2')*(qc.l('s3')-qc.l('s2'))/sumq23.l
                  +(rc.l('s2')-rc.l('s3'))*(5*qc.l('s2')-1)/sumr23.l
                  +5*rc.l('s2')*(qc.l('s2')-qc.l('s3'))/sumr23.l
                  +((rc.l('s2')-rc.l('s3'))*(rc.l('s2')-5*rc.l('s2')*(sumq23.l)))/power(sumr23.l,2);

th23.l(k)=(x23.l('s2')*Q(k)*n.l('s2',k)+(1-x23.l('s2'))*Q(k)*n.l('s3',k))/sumq23.l;

w23.l(k)=sum(m,th13.l(m)*ps(m,k));

d_th23.l(k)=((Q(k)*n.l('s2',k)- Q(k)*n.l('s3',k))*sumq23.l-(x23.l('s2')*Q(k)*n.l('s2',k)+ x23.l('s3')*Q(k)*n.l('s3',k))*(qc.l('s2')-qc.l('s3')))/power(sumq23.l,2);

d_w23.l(k)=sum(m,ps(m,k)*d_th23.l(m));

a23.l(k)=(d_th23.l(k)*w23.l(k)-th23.l(k)*d_w23.l(k))/power(w23.l(k),2);
c23.l(k)=d_w23.l(k)/w23.l(k);

d_lngr23.l('s2')=-sum(k,a23.l(k)*b12.l('s2',k)+Q(k)*n.l('s2',k)*c23.l(k));

d_lng23.l('s2')=d_lngc23.l('s2')+d_lngr23.l('s2');

MODEL N3 / ALL/;

$onecho > dicopt.opt
stop 0
maxcycles 400
$offecho

option nlp=conopt3;
option mip=cplex;
option rminlp=conopt3;
option minlp=dicopt;
N3.optfile=1;

option decimals=5;
OPTION OPTCA = 1e-10;
option iterlim = 1000000;
option optcr  = 1e-5;
OPTION reslim = 1000000;
OPTION DOMLIM = 8;

option sysout=on;
solve N3 using minlp maximising z;

parameter
stab12     'stabbility criterion'
stab13     'stabbility criterion'
stab23     'stabbility criterion'
;
stab12= d_lng12.l('s1')*x12.l('s1')+1;
stab13= d_lng13.l('s1')*x13.l('s1')+1;
stab23= d_lng23.l('s2')*x23.l('s2')+1;

display
z.l
y.l
x.l
Y1.l,Y2.l,Y3.l
stab12,stab13,stab23
;
